#include <stdio.h>
#include <stdlib.h>
#include <math.h>

//三次関数の設定
#define CUBIC_FUNC_A 0.4
#define CUBIC_FUNC_B -0.2
#define CUBIC_FUNC_C -1.0
#define CUBIC_FUNC_D 1.0

//作成する点のx座標の範囲(最小値、最大値を含む)
#define X_MIN -2.0
#define X_MAX 2.0

//作成する点の個数
#define NUM_POINTS 10000

//乱数のシード
#define SEED 1

//作成する点に与える誤差(正規分布)の分散
#define VARIANCE 0.00001

void print_matrix(double matrix[][4])
{
	for(int i = 0; i < 4; i++)
	{
		for(int j = 0; j < 4; j++)
			printf("%8.4f", matrix[j][i]);
		printf("\n");
	}
	printf("\n");	
}

void print_matrix_long(double matrix[][4])
{
	for(int i = 0; i < 4; i++)
	{
		for(int j = 0; j < 8; j++)
			printf("%12.4f", matrix[j][i]);
		printf("\n");
	}
	printf("\n");	
}

void print_array(double array[], int length)
{
	for(int i = 0; i < length; i++)
		printf("%8.4f\n", array[i]);
	printf("\n");
}

//三次関数の計算
double calc_func(double x)
{
	return CUBIC_FUNC_A * pow(x, 3.0) + CUBIC_FUNC_B * pow(x, 2.0) + CUBIC_FUNC_C * x + CUBIC_FUNC_D;
}

//0<f(x)<1の乱数を生成
double urand()
{
	return ((double)rand() + 1 ) / ((double)RAND_MAX - 2);
}

//正規分布を生成
double gauss_dis()
{
	return sqrt(-2 * log(urand())) * cos(2 * M_PI + urand()) * VARIANCE;
}

//点を計算
void generate_points(double *point_x, double *point_y, int num_points)
{
	//xの設定
	for(int i = 0; i < num_points; i++)
		point_x[i] = (double)rand() / (double)RAND_MAX * ((double)X_MAX - (double)X_MIN) + (double)X_MIN;
	//yの設定
	for(int i = 0; i < num_points; i++)
		point_y[i] =  gauss_dis() + calc_func(point_x[i]);
}

//sigma(x^)
double sigma_x(double *point_x, int expo, int num_points)
{
	double ans = 0.0;
	for(int i = 0; i < num_points; i++)
		ans += pow(point_x[i], (double)expo);
	return ans;
}

//sigma(yx^)
double sigma_yx(double *point_x, double *point_y, int expo, int num_points)
{
	double ans = 0.0;
	for(int i = 0; i < num_points; i++)
		ans += point_y[i] * pow(point_x[i], (double)expo);
	return ans;
}

//逆行列を求める
void inv(double before[][4], double after[][4], int num_points)
{
	//横2n縦nの行列
	double inter[4 * 2][4];

	//初期化
	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			inter[i][j] = before[i][j];
	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
		{
			if(i == j)
				inter[4 + i][j] = 1.0;
			else
				inter[4 + i][j] = 0.0;
		}
	
	printf("inv初期化\n");
	print_matrix_long(inter);
	
	//掃き出し
	for(int i = 0; i < 4; i++)
	{
		//ゼロ除算チェック
		if(inter[i][i] == 0.0)
		{
			//入れ替え先の行を探索
			int offset = 1;
			while(1)
			{
				if(inter[i][i + offset] != 0.0)
					break;
				else
				{
					offset++;
					if(i + offset == 4 - 1)
					{
						printf("inv not exist\n");
						exit(1);
					}
				}
			}

			//行の入れ替え
			double temp_array[4];
			for(int j = 0; j < 4 * 2; j++)
			{
				temp_array[j] = inter[j][i + offset];
				inter[j][i + offset] = inter[j][i];
				inter[j][i] = temp_array[j];
			}
		}
		
		printf("invゼロ除算\n");
		print_matrix_long(inter);

		//他の行を0にする
		for(int j = 0; j < 4; j++)
		{
			if(i == j)
				continue;
			else
			{
				double temp_coef = inter[i][j] / inter[i][i];
				for(int k = i; k < 4 * 2; k++)
					inter[k][j] -= temp_coef * inter[k][i];
			}
		}
		
		printf("inv他行ゼロ\n");
		print_matrix_long(inter);
		
		//自行を1にする
		for(int j = i + 1; j < 8; j++)
		{
			inter[j][i] /= inter[i][i];
		}
		inter[i][i] /= inter[i][i];
		
		printf("inv自行1\n");
		print_matrix_long(inter);
	}

	//出力
	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			after[i][j] = inter[i + 4][j];
}

//掛け算
void mul(double *ans, double *array, double matrix[][4], int num_points)
{
	for(int i = 0; i < 4; i++)
	{
		ans[i] = 0.0;
		for(int j = 0; j < 4; j++)
			ans[i] += array[i] * matrix[i][j];
	}
}

void lsmethod(double *coef, double *point_x, double *point_y, int num_points)
{
	double matrix[4][4];
	for(int i = 0; i < 4; i++)
		for(int j = 0; j < 4; j++)
			matrix[i][j] = sigma_x(point_x, 5 - i - j, num_points);
	
	printf("lsmethod1\n");
	print_array(point_x, num_points);
	print_array(point_y, num_points);
	print_matrix(matrix);
	
	double array[4];
	for(int i = 0; i < 4; i++)
		array[i] = sigma_yx(point_x, point_y, 3 - i, num_points);
	
	
	double matrix_inv[4][4];
	inv(matrix, matrix_inv, num_points);
	
	printf("lsmethod2\n");
	print_matrix(matrix_inv);

	mul(coef, array, matrix_inv, num_points);
}

int main()
{
	srand(SEED);

	double point_x[NUM_POINTS], point_y[NUM_POINTS];
	generate_points(point_x, point_y, NUM_POINTS);

	double coef[4];
	lsmethod(coef, point_x, point_y, NUM_POINTS);

	printf("%lf, %lf, %lf, %lf\n", coef[0], coef[1], coef[2], coef[3]);
}
