#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <curand.h>
#include <curand_kernel.h>

//5億程度であればオーバーフローしない(ネイピア数を掛けると約13億<2^31=約21億)
#define NUM_TRY 500000000

__global__ void napier(int *count, unsigned int seed)
{
	int num_thread = blockDim.x * blockIdx.x + threadIdx.x;
	if(NUM_TRY <= num_thread)
		return;
	double total = 0.0;
	curandState state;
	curand_init(seed, num_thread, 0, &state);
	while(total < 1.0)
	{
		total += curand_uniform(&state);
		atomicAdd(count, 1);
	}
}

int main()
{
	int count = 0;
	int *count_device;
	cudaMalloc((void **)&count_device, sizeof(int));
	cudaMemcpy(count_device, &count, sizeof(int), cudaMemcpyHostToDevice);
	int dimgrid = (int)((double)NUM_TRY / 1024) + 1;
	napier<<<dimgrid, 1024>>>(count_device, (unsigned int)time(NULL));
	cudaMemcpy(&count, count_device, sizeof(int), cudaMemcpyDeviceToHost);
	//printf("%d\n", count);
	printf("%1.8f\n", (double)count / NUM_TRY);
	return 0;
}
